from typing import List, Mapping, Optional

from vital.api.api import API


class Sleep(API):
    """Endpoints for getting sleep data."""

    def get(
        self,
        user_key: str,
        start_date: str,
        end_date: str,
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        GET Sleep data.
        """
        return self.client.get(
            f"/sleep/{user_key}",
            params={
                "start_date": start_date,
                "end_date": end_date,
                "provider": provider,
            },
        )

    def get_stream(self, sleep_id: str) -> Mapping[str, List[Mapping]]:
        """
        GET Sleep stream data.
        """
        return self.client.get(
            f"/sleep/{sleep_id}/stream/data",
        )



from vital.api.activity import Activity
from vital.api.body import Body
from vital.api.link import Link
from vital.api.provider_specific import ProviderSpecific
from vital.api.sleep import Sleep
from vital.api.user import User
from vital.api.vitals import Vitals
from vital.api.webhooks import Webhooks
from vital.api.workouts import Workouts

__all__ = [
    "Link",
    "Body",
    "Activity",
    "Sleep",
    "ProviderSpecific",
    "User",
    "Workouts",
    "Webhooks",
    "Vitals",
]



try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    import importlib_metadata  # type: ignore

try:
    __version__ = importlib_metadata.version("vital")
except Exception:
    __version__ = "Undefined"



from typing import Any


class BaseError(Exception):
    def __init__(
        self,
        message,
        type,
        code,
    ):
        super(BaseError, self).__init__(message)

        # In Python 3, the Exception class does not expose a `message`
        # attribute so we need to set it explicitly. See
        # https://www.python.org/dev/peps/pep-0352/#retracted-ideas.
        self.message = message

        self.type = type
        self.code = code


class VitalError(BaseError):
    def __init__(
        self,
        message,
        type,
        code,
        causes=None,
    ):
        super(VitalError, self).__init__(
            message,
            type,
            code,
        )
        self.causes = [
            VitalCause(
                cause["error_message"],
                cause["error_type"],
                cause["error_code"],
            )
            for cause in causes or []
        ]

    @staticmethod
    def from_response(response: Any, status_code: int) -> "VitalError":
        """
        Create an error of the right class from an API response.
        :param   response    dict        Response JSON
        """
        if not type(response) == dict:
            raise InvalidRequestError("Invalid request", "INVALID_REQUEST", 400)
        if not response.get("error_type"):
            return InvalidRequestError("Invalid request", "INVALID_REQUEST", 400)
        cls = VITAL_ERROR_TYPE_MAP.get(response["error_type"], VitalError)
        return cls(
            response["error_message"],
            response["error_type"],
            status_code,
            response.get("causes"),
        )


class VitalCause(BaseError):
    def __init__(
        self,
        message,
        type,
        code,
    ):
        super(VitalCause, self).__init__(
            message,
            type,
            code,
        )


class InvalidRequestError(VitalError):
    """The request is malformed and cannot be processed."""

    pass


class InvalidInputError(VitalError):
    """The request is correctly formatted, but the values are incorrect."""

    pass


class RateLimitExceededError(VitalError):
    """The request is valid but has exceeded established rate limits."""

    pass


class APIError(VitalError):
    """Planned maintenance or an API internal server error."""

    pass


class SignatureVerificationError(VitalError):
    def __init__(self, message: str, sig_header: str):
        super(SignatureVerificationError, self).__init__(
            message, "INVALID_REQUEST", None
        )
        self.sig_header = sig_header


VITAL_ERROR_TYPE_MAP = {
    "INVALID_REQUEST": InvalidRequestError,
    "INVALID_INPUT": InvalidInputError,
    "RATE_LIMIT_EXCEEDED": RateLimitExceededError,
    "API_ERROR": APIError,
}



from typing import Dict, List, Mapping

from vital.api.api import API
from vital.internal.webhook import Webhook
from vital.types import WebhookEventCodes, WebhookType


class Webhooks(API):
    """Endpoints for getting sleep data."""

    def get(self) -> List[Mapping]:
        """
        GET List of webhooks
        """
        return self.client.get("/webhooks/")

    def register(
        self, callback_url: str, verify_token: str, event_type: str
    ) -> Mapping[str, str]:
        """
        Register a webhook.
        :param str callback_url: Webhook url to to receive events.
        :param str verify_token: Random token to verify request.
        :param str event_type: Type of webhooks to receive.
        """
        return self.client.post(
            "/webhooks/",
            {
                "callback_url": callback_url,
                "verify_token": verify_token,
                "event_type": event_type,
            },
        )

    def delete(self, webhook_id: str) -> Mapping[str, str]:
        """
        DELETE Webhook.
        :param str webhook_id: Webhook id returned on creation.
        """
        return self.client.delete(
            f"/webhooks/{webhook_id}",
        )

    def test(
        self, event_code: WebhookEventCodes, webhook_type: WebhookType
    ) -> Mapping[str, str]:
        """
        Test a webhook event
        :param str event_code: Webhook event_code HISTORICAL_DATA_UPDATE or CREATED
        :param str verify_token: Webhook type ACTIVITY, SLEEP, BODY, WORKOUTS
        """
        return self.client.post(
            "/webhooks/test",
            {"event_code": event_code.value, "webhook_type": webhook_type.value},
        )

    @staticmethod
    def construct_webhook_event(
        payload: str, received_sig: str, webhook_secret: str
    ) -> Dict:
        """
        Parses Webhook events & validates request signature.
        :param str payload: Payload as json string.
        :param str received_sig: Received_signature from webhook event.
        :param str webhook_secret: Youre client webhook secret.
        """
        return Webhook.construct_event(payload, received_sig, webhook_secret)



from typing import Any, Mapping

import arrow
import jwt
from auth0.v3.authentication import GetToken

audiences = {
    "production": "https://api.tryvital.io",
    "prod": "https://api.tryvital.io",
    "sandbox": "https://api.sandbox.tryvital.io",
}

domains = {
    "production": "auth.tryvital.io",
    "prod": "auth.tryvital.io",
    "sandbox": "auth.sandbox.tryvital.io",
}


class TokenHandler:
    def __init__(
        self,
        client_id: str,
        client_secret: str,
        env: str,
        audience: str = None,
        domain: str = None,
    ) -> None:
        self.client_id = client_id
        self.client_secret = client_secret
        self.env = env
        self.audience = audiences[env] if not audience else audience
        self.domain = domains[env] if not domain else domain
        self._access_token = self.generate_access_token()

    def generate_access_token(self) -> Mapping[str, Any]:
        get_token = GetToken(self.domain)
        token = get_token.client_credentials(
            self.client_id, self.client_secret, self.audience
        )
        token = token["access_token"]
        token_details = jwt.decode(token, options={"verify_signature": False})
        return {"token": token, "exp": token_details["exp"]}

    @property
    def access_token(self) -> str:
        current_time = arrow.get().int_timestamp
        if current_time > self._access_token["exp"]:
            self._access_token = self.generate_access_token()
            return self._access_token["token"]
        return self._access_token["token"]



import requests

from vital.api import Activity, Body, Link, ProviderSpecific, Sleep, Webhooks, Workouts
from vital.api.user import User
from vital.api.vitals import Vitals
from vital.internal.requester import (
    DEFAULT_TIMEOUT,
    delete_request,
    get_request,
    post_request,
)
from vital.internal.token_handler import TokenHandler
from vital.internal.utils import urljoin

base_urls = {
    "prod": "https://api-tryvital-io.relay.evervault.com",
    "production": "https://api-tryvital-io.relay.evervault.com",
    "dev": "https://api-dev-tryvital-io.relay.evervault.com",
    "sandbox": "https://api-sandbox-tryvital-io.relay.evervault.com",
}


def get_base_url(environment: str) -> str:
    if environment == "local":
        return "http://localhost:8000"
    try:
        return base_urls[environment]
    except KeyError:
        raise Exception("Environment not supported")


class Client:
    """
    Python Vital API client.
    All of the endpoints documented under the ``vital.api``
    module may be called from a ``vital.Client`` instance.
    """

    def __init__(
        self,
        client_id=None,
        secret=None,
        environment=None,
        timeout=DEFAULT_TIMEOUT,
        api_version="v1",
        **kwargs,
    ):
        """
        Initialize a client with credentials.
        :param  str     client_id:          Your Vital client ID
        :arg    str     secret:             Your Vital secret
        :arg    str     environment:        One of ``sandbox`` or ``production``.
        :arg    int     timeout:            Timeout for API requests.
        """
        self.client_id = client_id
        self.client_secret = secret
        self.environment = environment
        self.timeout = timeout
        self.api_version = api_version
        self.base_url = get_base_url(environment)
        self.token_handler = TokenHandler(
            self.client_id,
            self.client_secret,
            self.environment,
            audience=kwargs.get("audience"),
            domain=kwargs.get("domain"),
        )
        self.session = requests.Session()
        # Mirror the HTTP API hierarchy
        self.Link = Link(self)
        self.Body = Body(self)
        self.Activity = Activity(self)
        self.ProviderSpecific = ProviderSpecific(self)
        self.Sleep = Sleep(self)
        self.User = User(self)
        self.Workouts = Workouts(self)
        self.Webhooks = Webhooks(self)
        self.Vitals = Vitals(self)

    def post(self, path, data, is_json=True, params={}, headers={}):
        """Make a post request."""
        return self._post(path, data, is_json, params, self.session, headers)

    def get(self, path, params={}, headers={}):
        """Make a get request."""
        return self._get(path, params, self.session, headers)

    def delete(self, path, params={}):
        """Make a delete request."""
        return self._delete(path, params, self.session)

    def post_public(self, path, data, is_json=True):
        """Make a post request requiring no auth."""
        return self._post(path, data, is_json, self.session)

    def _post(self, path, data, is_json, params={}, session=None, headers={}):
        headers = {
            "Authorization": f"Bearer {self.token_handler.access_token}",
            "Accept-Encoding": "deflate",
            **headers,
        }
        return post_request(
            urljoin(
                self.base_url,
                f"{self.api_version}{path}",
            ),
            data=data,
            timeout=self.timeout,
            is_json=is_json,
            headers=headers,
            params=params,
            session=session,
        )

    def _get(self, path, params={}, session=None, headers={}):
        headers = {
            "Authorization": f"Bearer {self.token_handler.access_token}",
            "Accept-Encoding": "deflate",
            **headers,
        }
        return get_request(
            urljoin(
                self.base_url,
                f"{self.api_version}{path}",
            ),
            timeout=self.timeout,
            headers=headers,
            params=params,
            session=session,
        )

    def _delete(self, path, params={}, session=None):
        headers = {
            "Authorization": f"Bearer {self.token_handler.access_token}",
            "Accept-Encoding": "deflate",
        }
        return delete_request(
            urljoin(
                self.base_url,
                f"{self.api_version}{path}",
            ),
            timeout=self.timeout,
            headers=headers,
            params=params,
            session=session,
        )



import hmac
import json
import secrets
import time
from hashlib import sha256
from typing import Dict, List, Optional, Tuple, Type, TypeVar

from vital.errors import SignatureVerificationError

T = TypeVar("T", bound="WebhookSignature")


def secure_compare(val1: str, val2: str) -> bool:
    return hmac.compare_digest(val1, val2)


class Webhook(object):
    DEFAULT_TOLERANCE = 300

    @staticmethod
    def construct_event(
        payload: str, sig_header: str, secret: str, tolerance: int = DEFAULT_TOLERANCE
    ) -> Dict:

        WebhookSignature.verify_header(payload, sig_header, secret, tolerance)

        data = json.loads(payload, object_pairs_hook=dict)
        return data


class WebhookSignature(object):
    EXPECTED_SCHEME = "v1"

    @staticmethod
    def _compute_signature(payload: str, secret: str) -> str:
        mac = hmac.new(
            secret.encode("utf-8"),
            msg=payload.encode("utf-8"),
            digestmod=sha256,
        )
        return mac.hexdigest()

    @staticmethod
    def _get_timestamp_and_signatures(
        header: str, scheme: str
    ) -> Tuple[int, List[str]]:
        list_items = [i.split("=", 2) for i in header.split(",")]
        timestamp = int([i[1] for i in list_items if i[0] == "t"][0])
        signatures = [i[1] for i in list_items if i[0] == scheme]
        return timestamp, signatures

    @classmethod
    def verify_header(
        cls: Type["T"],
        payload: str,
        header: str,
        secret: str,
        tolerance: Optional[int] = None,
    ) -> bool:
        try:
            timestamp, signatures = cls._get_timestamp_and_signatures(
                header, cls.EXPECTED_SCHEME
            )
        except Exception:
            raise SignatureVerificationError(
                "Unable to extract timestamp and signatures from header",
                header,
            )

        if not signatures:
            raise SignatureVerificationError(
                "No signatures found with expected scheme " "%s" % cls.EXPECTED_SCHEME,
                header,
            )

        signed_payload = "%d.%s" % (timestamp, payload)
        expected_sig = cls._compute_signature(signed_payload, secret)
        if not any(secure_compare(expected_sig, s) for s in signatures):
            raise SignatureVerificationError(
                "No signatures found matching the expected signature for " "payload",
                header,
            )

        if tolerance and timestamp < time.time() - tolerance:
            raise SignatureVerificationError(
                "Timestamp outside the tolerance zone (%d)" % timestamp,
                header,
            )

        return True


def generate_webhook_secret() -> str:
    return secrets.token_hex(32)



from enum import Enum


class WebhookType(Enum):
    ACTIVITY = "activity"
    SLEEP = "sleep"
    BODY = "body"
    WORKOUTS = "workouts"
    VITALS_GLUCOSE = "vitals_glucose"


class WebhookEventCodes(Enum):
    # Historical Data is now available
    HISTORICAL_DATA_UPDATE = "HISTORICAL_DATA_UPDATE"
    # New data has been created
    CREATED = "CREATED"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['vital', 'vital.api', 'vital.internal']

package_data = \
{'': ['*']}

install_requires = \
['PyJWT>=2.0.1,<3.0.0',
 'arrow',
 'auth0-python',
 'importlib-metadata>=3.7.3,<4.0.0',
 'requests']

setup_kwargs = {
    'name': 'vital',
    'version': '0.3.5',
    'description': '',
    'long_description': '# vital-python\n\nThe official Python Library for [Vital API](https://docs.tryvital.io)\n\n\n# Install\n```\npip install vital\n```\n\n# Calling Endpoints\n\n```\nfrom vital import Client\n\n# Available environments are \'sandbox\', \'development\', and \'production\'.\nclient = Client(client_id=\'***\', secret=\'***\', environment=\'sandbox\')\n```\n\n# Supported Endpoints\n\n```\n<!-- Dates have to be url encoded -->\nstart_date =  (datetime.now()-timedelta(days=1)).isoformat())\nend_date = datetime.now().isoformat()\n\nclient.Link.create(user_key="user_key")\nclient.Body.get(user_key=**,start_date, end_date)\nclient.Activity.get(user_key=**,start_date, end_date)\nclient.Sleep.get(user_key=**,start_date, end_date)\nclient.SourceSpecific.get(user_key=**,start_date, end_date)\nclient.User.create(client_user_id=**)\nclient.User.providers(user_key=**)\nclient.User.get(client_user_id=**)\n\n\nfrom vital.types import WebhookEventCodes, WebhookType\n\nclient.Webhooks.test(WebhookEventCodes.HISTORICAL_DATA_UPDATE, \n                     WebhookType.ACTIVITY)\n```\n\n\n# Installing locally \n```\npoetry build --format sdist\ntar -xvf dist/*-`poetry version -s`.tar.gz -O \'*/setup.py\' > setup.py\n```',
    'author': 'maitham',
    'author_email': 'maitham@tryvital.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/adeptlabs/vital-python',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import List, Mapping, Optional

from vital.api.api import API


class ProviderSpecific(API):
    """Endpoints for getting provider specific data."""

    def get(
        self,
        user_key: str,
        provider: str,
        start_date: str,
        end_date: str,
        data_type: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        Get Provider Specific data
        :param str user_key: users key
        :param str start_date: date in ISO format
        :param str end_date: date in ISO format
        :param Optional[str] provider: Provider of data strava etc.
        :param Optional[str] data_type: activity | sleep | body | workouts
        """
        return self.client.get(
            f"/provider-specific/{user_key}/{provider}",
            params={
                "start_date": start_date,
                "end_date": end_date,
                "data_type": data_type,
            },
        )



from typing import List, Mapping, Optional

from vital.api.api import API


class Vitals(API):
    """Endpoints for getting activity data."""

    def glucose(
        self,
        user_key: str,
        start_date: str,
        end_date: str,
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        Get glucose value API
        :param str user_key: users key
        :param str start_date: date in ISO format
        :param str end_date: date in ISO format
        :param Optional[str] provider: Provider of data
        """
        return self.client.get(
            f"/vitals/glucose/{user_key}",
            params={
                "start_date": start_date,
                "end_date": end_date,
                "provider": provider,
            },
        )

    def hba1c(
        self,
        user_key: str,
        start_date: str,
        end_date: str,
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        Get hba1c value API
        :param str user_key: users key
        :param str start_date: date in ISO format
        :param str end_date: date in ISO format
        :param Optional[str] provider: Provider of data
        """
        return self.client.get(
            f"/vitals/hba1c/{user_key}",
            params={
                "start_date": start_date,
                "end_date": end_date,
                "provider": provider,
            },
        )

    def cholesterol(
        self,
        type: str,
        user_key: str,
        start_date: str,
        end_date: str,
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        Get glucose value API
        :param str user_key: users key
        :param str start_date: date in ISO format
        :param str end_date: date in ISO format
        :param Optional[str] provider: Provider of data
        """
        return self.client.get(
            f"/vitals/cholesterol/{type}/{user_key}",
            params={
                "start_date": start_date,
                "end_date": end_date,
                "provider": provider,
            },
        )

    def ige(
        self,
        user_key: str,
        start_date: str,
        end_date: str,
        provider: Optional[str] = "",
    ) -> Mapping[str, List[Mapping]]:
        """
        Get glucose value API
        :param str user_key: users key
        :param str start_date: date in ISO format
        :param str end_date: date in ISO format
        """
        return self.client.get(
            f"/vitals/ige/{user_key}",
            params={"start_date": start_date, "end_date": end_date},
        )

    def igg(
        self,
        user_key: str,
        start_date: str,
        end_date: str,
    ) -> Mapping[str, List[Mapping]]:
        """
        Get glucose value API
        :param str user_key: users key
        :param str start_date: date in ISO format
        :param str end_date: date in ISO format
        """
        return self.client.get(
            f"/vitals/igg/{user_key}",
            params={
                "start_date": start_date,
                "end_date": end_date,
            },
        )



from typing import List, Mapping

from vital.api.api import API

link_token_field_names = [
    "client_user_id",
]


class User(API):
    """Endpoints for managing link tokens."""

    def create(self, client_user_id: str) -> Mapping[str, str]:
        """
        Create a Link token.
        :param str client_user_id: A non phi user_id.
        """

        return self.client.post("/user/key", {"client_user_id": client_user_id})

    def delete(self, user_key: str) -> Mapping[str, str]:
        """
        Delete user and associated data this is irreversible.
        :param str user_key: Provided user_key
        """

        return self.client.delete(f"/user/{user_key}")

    def get_all(self) -> Mapping[str, str]:
        """
        Get all users.
        """
        return self.client.get("/user/")

    def get(self, user_key: str) -> Mapping[str, str]:
        """
        Get user id.
        :param str user_key: The client user id.
        """
        return self.client.get(f"/user/{user_key}")

    def resolve(self, client_user_id: str) -> Mapping[str, str]:
        """
        Get user details from key.
        :param str client_user_id: The client user id.
        """
        return self.client.get(f"/user/key/{client_user_id}")

    def providers(self, user_key: str) -> List[Mapping[str, str]]:
        """
        Get list of providers.
        :param str user_key: User key provided by organisation.
        """

        return self.client.get(f"/user/providers/{user_key}")

    def deregister_provider(
        self, user_key: str, provider: str
    ) -> List[Mapping[str, str]]:
        """
        Deregister provider.
        :param str user_key: The generated user_key
        :param str provider: Provider to deregister
        """

        return self.client.delete(f"/user/{user_key}/{provider}")
